<?php
/**
* Plugin Name: SEORadar
* Plugin URI: http://www.seoradar.com/
* Description: SEORadar monitors your site for SEO issues and generates alerts when problems are detected.
* Version: 1.0
* Author: Mark Munroe
* Author URI: https://www.linkedin.com/in/munroeseo
* License: GPL2
*/
/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License, version 2, as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

define("__SEO_RADAR_PLUGIN_NAME__", "SEORadar");
define("__SEO_RADAR_PLUGIN_SLUG__", "__seo_radar_");
define("__SEO_RADAR_VERSION__", 1.0);
define("__SEO_RADAR_DIR__", trailingslashit(plugin_dir_path(__FILE__)));
define("__SEO_RADAR_URL__", plugin_dir_url(__FILE__));
define("__SEO_RADAR_ROOT__", trailingslashit(plugins_url("", __FILE__)));
define("__SEO_RADAR_RESOURCES__", __SEO_RADAR_ROOT__ . "resources/");
define("__SEO_RADAR_IMAGES__", __SEO_RADAR_RESOURCES__ . "images/");
define("__SEO_RADAR_DEBUG__", false);
define("__SEO_RADAR_TEST__", false);
define("__SEO_RADAR_STAGING__", false);

define("__SEO_RADAR_API_ENDPOINT__", "api/v1/website_urls");
define("__SEO_RADAR_API_URL__", trailingslashit(__SEO_RADAR_STAGING__ ? "http://staging.seoradar.com/" : "https://app.seoradar.com/") . __SEO_RADAR_API_ENDPOINT__);

if(__SEO_RADAR_DEBUG__){
    @error_reporting(E_ALL);
    @ini_set("display_errors", "1");
}

/**
 * Abort loading if WordPress is upgrading
 */
if (defined("WP_INSTALLING") && WP_INSTALLING) return;

class SEORadar{

    private $error;
    private $notice;

    public function __construct(){
        // all hooks and actions
        register_activation_hook(__FILE__ , array($this, "seo_radar_activate"));
        register_deactivation_hook(__FILE__ , array($this, "seo_radar_deactivate"));

        add_action("init", array($this, "seo_radar_register"));
        add_action("admin_enqueue_scripts", array($this, "seo_radar_includeResources"));
        add_action("plugins_loaded", array($this, "seo_radar_i18n"));
        add_action("admin_menu", array($this, "seo_radar_add_menu"));
        add_action("post_submitbox_misc_actions", array($this, "seo_radar_showExtra"));
        add_action("save_post", array($this, "seo_radar_savePost"));
        add_action("admin_notices", array($this, "seo_radar_adminNotice"));
        add_action("load-edit.php", array($this, "seo_radar_post_listing"));
        //add_action("parse_request", array($this, "seo_radar_parse_request"));

    }

    function seo_radar_parse_request($query){
        self::writeDebug("query = " . print_r($query, true) . " uri = " . $_SERVER["REQUEST_URI"]);
    }

    /**
     * Initializes the locale
     */
    function seo_radar_i18n(){
        $pluginDirName  = dirname(plugin_basename(__FILE__));
        $domain         = __SEO_RADAR_PLUGIN_SLUG__;
        $locale         = apply_filters("plugin_locale", get_locale(), $domain);
        load_textdomain($domain, WP_LANG_DIR . "/" . $pluginDirName . "/" . $domain . "-" . $locale . ".mo");
        load_plugin_textdomain($domain, "", $pluginDirName . "/resources/lang/");
    }

    function seo_radar_footer(){
        include_once __SEO_RADAR_DIR__ . "resources/templates/footer.php";
    }

    /**
     * Initializes the admin menu
     */
    function seo_radar_add_menu(){
        add_menu_page(__SEO_RADAR_PLUGIN_NAME__, __SEO_RADAR_PLUGIN_NAME__, "manage_options", __SEO_RADAR_PLUGIN_SLUG__, array($this, "seo_radar_settings"), __SEO_RADAR_IMAGES__ . "favicon.png");
    }

    /**
     * Saves settings from the settings screen
     */
    function seo_radar_settings(){
        global $hasCurl;
        $hasCurl    = function_exists("curl_exec");

        if(isset($_POST["sr-submit"])){
            self::saveSettings();
        }
        include_once __SEO_RADAR_DIR__ . "resources/admin/includes/settings.php";
    }

    /**
     * Loads the JS and CSS resources
     */
    function seo_radar_includeResources() {
        wp_enqueue_script("jquery");

        wp_register_script("seoradar", __SEO_RADAR_RESOURCES__ . "admin/js/seoradar.js");
        wp_enqueue_script("seoradar");

        wp_register_style("seoradar", __SEO_RADAR_RESOURCES__ . "admin/css/seoradar.css");
        wp_enqueue_style("seoradar");
    }

    /**
     * Register the plugin
     */
    function seo_radar_register(){
        // do nothing
    }

    /**
     * Activate the plugin
     */
    function seo_radar_activate(){
        if(function_exists("curl_exec")){
            self::setOption("share", 1);
        }else{
            @session_start();
            $_SESSION[__SEO_RADAR_PLUGIN_SLUG__ . "error"] = __("Please activate PHP cURL", __SEO_RADAR_PLUGIN_SLUG__);
            add_action("admin_notices", array($this, "seo_radar_adminNotice"));
        }
    }

    function seo_radar_showExtra(){
        global $post, $pagenow;
        if(self::getOption("share") == 1 && ($post->post_type == "post" || $post->post_type == "page" || $pagenow == "post-new.php")){
            @session_start();
            include_once __SEO_RADAR_DIR__ . "resources/admin/includes/extra.php";
        }
    }

    /**
     * Deactivate the plugin
     */
    function seo_radar_deactivate(){
        if(__SEO_RADAR_TEST__){
            define("WP_UNINSTALL_PLUGIN", true);
            include_once __SEO_RADAR_DIR__ . "uninstall.php";
        }
    }

    function seo_radar_post_listing(){
        if(@intval($_GET["trashed"]) > 0 && @isset($_GET["ids"])){
            $this->deletePost(explode(",", $_GET["ids"]));
        }
    }

    private function deletePost($postIDs){
        $urls           = array();
        foreach($postIDs as $id){
            $url        = rawurlencode(get_permalink($id));
            $urls[]     = $url;
        }
        $params     = array("url" => implode(",", $urls));
        
        $this->callAPI($params, true);
    }

    function seo_radar_adminNotice(){
        @session_start();
        if(isset($_SESSION[__SEO_RADAR_PLUGIN_SLUG__ . "error"])){
            echo '<div class="error seoradar"><p>' . __SEO_RADAR_PLUGIN_NAME__ . ": " . __($_SESSION[__SEO_RADAR_PLUGIN_SLUG__ . "error"], __SEO_RADAR_PLUGIN_SLUG__) . "</p></div>";
            unset($_SESSION[__SEO_RADAR_PLUGIN_SLUG__ . "error"]);
        }
    }

    function seo_radar_savePost($postID){
        if(!isset($_POST["ping_seo_radar"])) return;

        $post       = get_post($postID);
        if($post->post_status != "publish") return;

        $url        = rawurlencode(get_permalink($postID));
        $params     = array("url" => $url);
        $this->getFocusKeyword($params, $postID);
    
        $this->callAPI($params);
    }

    private function getFocusKeyword(&$array, $postID){
        if(is_plugin_active("wordpress-seo/wp-seo.php")){
            $keyword    = isset($_REQUEST["yoast_wpseo_focuskw"]) ? $_REQUEST["yoast_wpseo_focuskw"] : null;
            if($keyword){
                $array["focus_keywords"]    = rawurlencode($keyword);
            }
        }
    }

    function saveSettings(){
        self::setOption("share", @$_POST["share"]);
        self::setOption("api-key", @$_POST["api-key"]);
    }

    function callAPI($params, $delete=FALSE){
        $token  = self::getOption("api-key");

        $url    = __SEO_RADAR_API_URL__ . ($delete ? "/destroy" : "");

        $conn   = curl_init($url);
        curl_setopt($conn, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($conn, CURLOPT_FRESH_CONNECT, true);
        curl_setopt($conn, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($conn, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($conn, CURLOPT_CONNECTTIMEOUT, 0);
        curl_setopt($conn, CURLOPT_HEADER, 0);
        curl_setopt($conn, CURLOPT_HTTPHEADER, array("X-Auth-Token: $token"));
        curl_setopt($conn, CURLOPT_POSTFIELDS, urldecode(http_build_query($params)));

        if(__SEO_RADAR_DEBUG__){
            curl_setopt($conn, CURLOPT_VERBOSE, 1);
            $verbose = fopen('php://temp', 'w+');
            curl_setopt($conn, CURLOPT_STDERR, $verbose);
        }

        if($delete){
            curl_setopt($conn, CURLOPT_CUSTOMREQUEST, "DELETE");
        }else{
            curl_setopt($conn, CURLOPT_POST, 1);
        }
        try{
            $response   = curl_exec($conn);
            $error      = curl_getinfo($conn, CURLINFO_HTTP_CODE);
        }catch(Exception $e){
        }
        if(curl_errno($conn)){
            self::writeDebug("curl_errno ".curl_error($conn));
        }

        if(__SEO_RADAR_DEBUG__){
            rewind($verbose);
            $verboseLog = stream_get_contents($verbose);
            self::writeDebug("cURL " . htmlspecialchars($verboseLog));
        }
        curl_close($conn);

        self::writeDebug("Calling ".$url. " with fields = " . print_r($params, true) . " response = ".$response);

        if($error != 200){
            @session_start();
            $array          = json_decode($response, true);
            if(is_array($array)){
                $_SESSION[__SEO_RADAR_PLUGIN_SLUG__ . "error"]  = $array["error"]["message"];
                if(__SEO_RADAR_TEST__){
                    $_SESSION[__SEO_RADAR_PLUGIN_SLUG__ . "error"]  = "API: " . $response;
                }
            }else{
                $_SESSION[__SEO_RADAR_PLUGIN_SLUG__ . "error"]  = __("Error attempting to connect to SEORadar.  Please try saving one more time.", __SEO_RADAR_PLUGIN_SLUG__);
            }
        }
    }

    /****************************************** Util functions ******************************************/

    /**
     * Writes to the file /tmp/log.log if DEBUG is on
     */
    public static function writeDebug($msg){
        if(__SEO_RADAR_DEBUG__){
            file_put_contents(__SEO_RADAR_DIR__ . "tmp/log.log", "------------------------------------------------\n", FILE_APPEND);
            file_put_contents(__SEO_RADAR_DIR__ . "tmp/log.log", date("F j, Y H:i:s") . " - " . $msg . "\n", FILE_APPEND);
        }
    }

    /**
     * Custom wrapper for the get_option function
     * 
     * @return string
     */
    public static function getOption($field, $clean=false){
        $val = get_option(__SEO_RADAR_PLUGIN_SLUG__ . $field);
        return $clean ? htmlspecialchars($val) : $val;
    }

    /**
     * Custom wrapper for the update_option function
     * 
     * @return mixed
     */
    public static function setOption($field, $value){
        return update_option(__SEO_RADAR_PLUGIN_SLUG__ . $field, $value);
    }

    /**
     * Custom wrapper for the get_post_meta function
     * 
     * @return mixed
     */
    public static function getPostMeta($postID, $name, $single=true){
        return get_post_meta($postID, __SEO_RADAR_PLUGIN_SLUG__ . $name, $single);
    }

    /**
     * Custom wrapper for the update_post_meta function
     */
    public static function setPostMeta($postID, $name, $value){
        update_post_meta($postID, __SEO_RADAR_PLUGIN_SLUG__ . $name, $value);
    }

}

$seoRadar = new SEORadar();
